@@ -1,5 +1,5 @@
-from mcstatus.pinger import ServerPinger
-from mcstatus.protocol.connection import TCPSocketConnection, UDPSocketConnection
+from mcstatus.pinger import ServerPinger, AsyncServerPinger
+from mcstatus.protocol.connection import TCPSocketConnection, UDPSocketConnection, TCPAsyncSocketConnection
 from mcstatus.querier import ServerQuerier
 from mcstatus.scripts.address_tools import parse_address
 import dns.resolver
@@ -26,10 +26,10 @@ class MinecraftServer:
 
         return MinecraftServer(host, port)
 
-    def ping(self, retries=3, **kwargs):
+    def ping(self, tries=3, **kwargs):
         connection = TCPSocketConnection((self.host, self.port))
         exception = None
-        for attempt in range(retries):
+        for attempt in range(tries):
             try:
                 pinger = ServerPinger(connection, host=self.host, port=self.port, **kwargs)
                 pinger.handshake()
@@ -39,10 +39,23 @@ class MinecraftServer:
         else:
             raise exception
 
-    def status(self, retries=3, **kwargs):
+    async def async_ping(self, tries=3, **kwargs):
+        connection = await TCPAsyncSocketConnection((self.host, self.port))
+        exception = None
+        for attempt in range(tries):
+            try:
+                pinger = AsyncServerPinger(connection, host=self.host, port=self.port, **kwargs)
+                pinger.handshake()
+                return await pinger.test_ping()
+            except Exception as e:
+                exception = e
+        else:
+            raise exception
+
+    def status(self, tries=3, **kwargs):
         connection = TCPSocketConnection((self.host, self.port))
         exception = None
-        for attempt in range(retries):
+        for attempt in range(tries):
             try:
                 pinger = ServerPinger(connection, host=self.host, port=self.port, **kwargs)
                 pinger.handshake()
@@ -54,7 +67,23 @@ class MinecraftServer:
         else:
             raise exception
 
-    def query(self, retries=3):
+    async def async_status(self, tries=3, **kwargs):
+        connection = TCPAsyncSocketConnection()
+        await connection.connect((self.host, self.port))
+        exception = None
+        for attempt in range(tries):
+            try:
+                pinger = AsyncServerPinger(connection, host=self.host, port=self.port, **kwargs)
+                pinger.handshake()
+                result = await pinger.read_status()
+                result.latency = await pinger.test_ping()
+                return result
+            except Exception as e:
+                exception = e
+        else:
+            raise exception
+
+    def query(self, tries=3):
         exception = None
         host = self.host
         try:
@@ -64,7 +93,7 @@ class MinecraftServer:
                 host = str(answer).rstrip(".")
         except Exception as e:
             pass
-        for attempt in range(retries):
+        for attempt in range(tries):
             try:
                 connection = UDPSocketConnection((host, self.port))
                 querier = ServerQuerier(connection)
@@ -74,3 +103,6 @@ class MinecraftServer:
                 exception = e
         else:
             raise exception
+
+    async def async_query(self, tries=3):
+        raise NotImplementedError # TODO: '-'
\ No newline at end of file
